/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.    
 */

package org.apache.tuscany.sca.contribution.jee;

import java.net.URI;
import java.util.HashMap;
import java.util.Map;

import org.apache.tuscany.sca.contribution.Artifact;
import org.apache.tuscany.sca.contribution.Contribution;
import org.apache.tuscany.sca.contribution.ModelFactoryExtensionPoint;
import org.apache.tuscany.sca.contribution.jee.impl.ModelObjectImpl;
import org.apache.tuscany.sca.contribution.resolver.ModelResolver;

/**
 * A Model Resolver for obtaining any model object using URI.
 *
 * @version $Rev: 768567 $ $Date: 2009-04-25 18:17:11 +0100 (Sat, 25 Apr 2009) $
 */
public class ModelObjectModelResolver implements ModelResolver {

    private Map<URI, ModelObject> map = new HashMap<URI, ModelObject>();
    private Contribution contribution;
    
    public ModelObjectModelResolver(Contribution contribution, ModelFactoryExtensionPoint modelFactories) {
        this.contribution = contribution;
    }

    public void addModel(Object resolved) {
        map.put(((ModelObject)resolved).getUri(), (ModelObject)resolved);
    }
    
    public Object removeModel(Object resolved) {
        return map.remove(((ModelObject)resolved).getUri());
    }
    
    public <T> T resolveModel(Class<T> modelClass, T unresolved) {
        URI uri = ((ModelObject)unresolved).getUri();
        if (uri != null) {
            ModelObject resolved = (ModelObject) map.get(uri);
            if (resolved != null) {
                return modelClass.cast(resolved);
            } else {
                for(Artifact artifact : contribution.getArtifacts()) {
                    if(artifact.getURI().equals(uri.toString())) {
                        resolved = new ModelObjectImpl();
                        resolved.setUri(uri);
                        resolved.setObject(artifact.getModel());
                        return modelClass.cast(resolved);
                    }
                }
            }
        }
        return unresolved;
    }
}
